#!/usr/bin/python
# -*- coding:utf-8 -*-

import string, re
from dommodel import *

url_info = "http://company.nikkei.co.jp/index.cfm?scode=4753"
url_chart_min = "http://chart.nikkei.co.jp/c/tick/maketick.cgi?STK_CODE=4753&BA_NUM=1"
url_chart_day = "http://chart.nikkei.co.jp/gif/ja/11d/4753.gif"


class NoCodeException ( Exception ):
    def __init__( self, message ):
        self.message = message
        pass
    def __repr__( self ):
        return "NO DATA"
    pass

class NikkeiStockInformation:
    THRESHOLD_CAUTION = 0.05
    def __init__( self, document ):
        self.__company = "company"
        self.__code    = "stock code"
        self.__place   = "exchange place"
        self.__field   = "category"
        self.__kessan  = "account term"
        self.__unit    = "exchange unit"
        
        self.__price_start = "start price"
        self.__price_high  = "high price"
        self.__price_low   = "low price"
        self.__price_now   = "current price"
        self.__price_difference = "trend from the day before"
        self.__price_quantity   = "exchanged quantity"
        self.__ba_num = 1

        self.__doc = document
        self.__parse( document )
        pass

    def __read_basic_information( self, doc ):
        elem_title = doc.get_element_at_path( "/html/body/div/h2" )
        if elem_title.get_element_at_path( "p/span" ) == None:
            raise NoCodeException( "no data" )
            
        self.__company = elem_title.get_first_child().get_text()
        self.__place   = elem_title.get_element_at_path( "p/strong" ).get_text()

        elem_info = elem_title.get_parent().get_element_at_path( "p/span" )
        self.__code = string.atoi( elem_info.get_element_at_path( "strong" ).get_text() )

        elem_info = elem_info.next_sibling( "span" )
        self.__kessan = elem_info.get_element_at_path( "strong" ).get_text()

        elem_info = elem_info.next_sibling( "span" )
        self.__unit = elem_info.get_element_at_path( "strong" ).get_text()
        
        elem_info = elem_info.next_sibling( "span" )
        self.__field = elem_info.get_element_at_path( "strong" ).get_text()
        pass

    def __read_prices( self, doc ):
        elem_price = doc.get_element_at_path( "/html/body/div/table" ).next_sibling( "table" ).next_sibling( "table" ).get_element_at_path( "tr/td" )
        self.__price_start = elem_price.get_text()
        elem_price = elem_price.next_sibling()
        self.__price_high  = elem_price.get_text()
        elem_price = elem_price.next_sibling()
        self.__price_low  = elem_price.get_text()
        elem_price = elem_price.next_sibling()
        self.__price_now  = elem_price.get_text()
        elem_price = elem_price.next_sibling()
        self.__price_difference  = re.sub( "\\s+", " ", elem_price.get_text() )
        elem_price = elem_price.next_sibling()
        self.__price_quantity  = elem_price.get_text()
        pass

    def __read_ba_num( self, doc ):
        elem = doc.get_element_at_path( "/html/body/div/table/tr/td/div/p/a" )
        if elem:
            att = elem.get_attribute( "href" )
            if att:
                m = re.search( "ba=(\\d+)", att )
                if m: self.__ba_num = string.atoi( m.group( 1 ) )
                pass
            pass
        return
        
    def __parse( self, doc ):
        try:
            # basic information
            self.__read_basic_information( doc )
            
            # current stock price
            self.__read_prices( doc )

            # read section
            self.__read_ba_num( doc )
        except NoCodeException, e:
            raise e
        except Exception, e:
            print e
            pass

        pass

    def __repr__( self ):
        line = ""
        line += "NAME: %s\n" % self.__company
        line += "CODE: %d\n" % self.__code
        line += "PLAC: %s\n" % self.__place
        line += "CTGR: %s\n" % self.__field
        line += "TERM: %s\n" % self.__kessan
        line += "UNIT: %s\n" % self.__unit
        line += "\n"
        line += "STRT: %s\n" % self.__price_start
        line += "HIGH: %s\n" % self.__price_high
        line += "LOWP: %s\n" % self.__price_low
        line += "CRRN: %s\n" % self.__price_now
        line += "TRND: %s\n" % self.__price_difference
        line += "QUNT: %s\n" % self.__price_quantity
        line += "\n"
        return line

    def __calculate_difference( self ):
        try:
            price_diff  = string.atoi( re.sub( "[^\\-\\d]", "", self.__price_difference ) )
            price_now = string.atoi( re.sub( "[^\\-\\d]", "", self.__price_now ) )
            return float( price_diff ) / ( price_now - price_diff );
        except:
            return 0.0
        pass

    def convert_to_html( self ):
        ratio = self.__calculate_difference()
        style = ""
        if ratio >= self.THRESHOLD_CAUTION:
            style = " style=\"color:#3f3;\""
        elif ratio <= - self.THRESHOLD_CAUTION:
            style = " style=\"color:#f33;\""
            pass
        
        line  = '<!--BA:%d-->' % self.__ba_num
        line += '<table id="infotable">'
        line += "<tr><th class=\"companyname\">会社名</th><th>株式コード</th><th>業種</th><th>決算期</th><th>売買単位</th><th>取引所</th></tr>"
        line += "<tr><td class=\"companyname\">%s</td><td>%s</td><td>%s</td><td>%s</td><td>%s</td><td>%s</td></tr>" % ( self.__company, self.__code, self.__field, self.__kessan, self.__unit, self.__place )
        line += '</table><table id="pricetable">'
        line += "<tr><th>始値</th><th>高値</th><th>安値</th><th>現在</th><th>前日比</th><th>取引量</th></tr>"
        line += "<tr><td>%s</td><td>%s</td><td>%s</td><td>%s</td><td%s>%s</td><td>%s</td></tr>" \
                % ( self.__price_start, self.__price_high, self.__price_low, self.__price_now, style, self.__price_difference, self.__price_quantity )
        line += "</table>"

        #line = "%s (%d) : %s" % ( self.__company, self.__code, self.__place )
        
        return line
    pass


if __name__ == "__main__":
    import tempfile, sys, os

    if len( sys.argv ) < 2: sys.exit()
    code = sys.argv[ 1 ]
    url = "http://company.nikkei.co.jp/index.cfm?scode=%s" % code
    p = DOMConverter()
    filename_temp = tempfile.mktemp( ".html", "stock" )
    os.system( '/usr/bin/curl "%s" -o %s -s' % ( url, filename_temp ) )
    if os.path.exists( filename_temp ) == False:
       print "CANNOT LOAD"
       sys.exit()
       pass
    data = ""
    for line in open( filename_temp ).readlines():
        p.feed( unicode( line, 'sjis' ).encode( 'utf-8' ) )
        pass
    os.unlink( filename_temp )
    #reader = urllib.urlopen( url )
    #data = reader.read()
    #p.feed( unicode( data, 'sjis' ).encode( 'utf-8' ) )
    doc = p.get_document()
    
    try:
        ni = NikkeiStockInformation( doc )
        output = ni.convert_to_html()
        print output
    except NoCodeException:
        print "NO CODE"
    except:
        pass
    pass


